from typing import Optional
from rest_framework import serializers
from counter.models import DeviceCount, DeviceInfo
from mosquito.models import DeviceInfo as MosquitoDeviceInfo


class DeviceInfoMobileSerializer(serializers.ModelSerializer):
    device_name = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()
    csq = serializers.SerializerMethodField()
    count = serializers.SerializerMethodField()
    vol = serializers.SerializerMethodField()
    coordinate = serializers.SerializerMethodField()
    location = serializers.SerializerMethodField()

    class Meta:
        model = DeviceInfo
        fields = [
            'device_id',
            'device_name',
            'status',
            'count',
            'csq',
            'vol',
            'coordinate',
            'location',
        ]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.latest = None
        self.cur_device = None
        self.counter_device = None

    def get_device_name(self, obj) -> Optional[str]:
        qs = MosquitoDeviceInfo.objects.filter(device_id=obj.device_id)
        if qs.count() > 0:
            device = qs.first()
            self.cur_device = device
            return device.device_name
        return None

    def get_status(self, obj) -> int:
        return obj.online

    def get_coordinate(self, obj):
        if self.cur_device:
            lon, lat = [self.cur_device.longitude, self.cur_device.latitude]
            if lon and lat:
                return [lon, lat]
        return None

    def get_location(self, obj):
        if self.cur_device:
            return self.cur_device.location
        return None

    def get_count(self, obj):
        device = DeviceCount.objects.filter(
            device_id=self.cur_device.device_id).order_by('-data_time').first()
        if device:
            self.counter_device = device
            return device.count
        return None

    def get_csq(self, obj):
        if self.counter_device:
            return self.counter_device.csq
        return None

    def get_vol(self, obj):
        if self.counter_device:
            return self.counter_device.vol
        return None
